@@ -1,9 +1,9 @@
 /*
 *
 * kkstrtext string related and text processing routines
-* $Id: kkstrtext.cc,v 1.31 2002/12/12 14:26:06 konst Exp $
+* $Id: kkstrtext.cc,v 1.38 2004/06/24 08:31:22 konst Exp $
 *
-* Copyright (C) 1999-2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 1999-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 */
 
 #include "kkstrtext.h"
+#include <errno.h>
 
 char *strcut(char *strin, int frompos, int count) {
     if(count > 0) {
@@ -763,12 +764,15 @@ string strdateandtime(struct tm *tms, const string &fmt) {
     string afmt = fmt;
 
     if(afmt.empty()) {
-	if(current_time > when + 6L * 30L * 24L * 60L * 60L /* Old. */
-	|| current_time < when - 60L * 60L)                 /* Future. */ {
+	afmt = "%b %e %Y %H:%M";
+/*
+	if(current_time > when + 6L * 30L * 24L * 60L * 60L // Old.
+	|| current_time < when - 60L * 60L) {               // Future.
 	    afmt = "%b %e  %Y";
 	} else {
 	    afmt = "%b %e %H:%M";
 	}
+*/
     }
 
     strftime(buf, 512, afmt.c_str(), tms);
@@ -785,7 +789,7 @@ bool iswholeword(const string &s, int so, int eo) {
     return rm && lm;
 }
 
-int hex2int(const string ahex) {
+int hex2int(const string &ahex) {
     int r, i;
 
     r = 0;
@@ -877,48 +881,51 @@ string ruscase(const string &s, const string &mode) {
     return r;
 }
 
-string siconv(const string &text, const string &fromcs, const string &tocs) {
-#ifdef HAVE_ICONV_H
+string siconv(const string &atext, const string &fromcs, const string &tocs) {
+#ifdef HAVE_ICONV
     iconv_t cd = iconv_open(tocs.c_str(), fromcs.c_str());
 
     if(((int) cd) != -1) {
-	string r;
+	string r, text(atext);
 	size_t inleft, outleft, soutleft;
 	char *inbuf, *outbuf, *sinbuf, *soutbuf;
 
-	sinbuf = inbuf = strdup(text.c_str());
-	inleft = strlen(inbuf);
+	while(!text.empty()) {
+	    sinbuf = inbuf = strdup(text.c_str());
+	    inleft = strlen(inbuf);
 
-	soutleft = outleft = inleft*4;
-	soutbuf = outbuf = new char[outleft];
+	    soutleft = outleft = inleft*4;
+	    soutbuf = outbuf = new char[outleft];
 
-	size_t res = iconv(cd,
-#ifdef ICONV_CONST
-	    (const char **)
-#endif
-	    &inbuf, &inleft, &outbuf, &outleft);
+	    size_t res = iconv(cd, (ICONV_CONST char **) &inbuf, &inleft,
+		&outbuf, &outleft);
 
-	iconv_close(cd);
-
-	if(res != -1) {
 	    soutbuf[soutleft-outleft] = 0;
-	    r = soutbuf;
-	} else {
-	    r = text;
+	    r += soutbuf;
+	    text.erase(0, text.size()-inleft);
+
+	    delete soutbuf;
+	    delete sinbuf;
+
+	    if(res == -1 && errno != EILSEQ)
+		break;
+
+	    if(!text.empty()) {
+		text.erase(0, 1);
+		r += " ";
+	    }
 	}
 
-	delete soutbuf;
-	delete sinbuf;
-
+	iconv_close(cd);
 	return r;
     }
 #endif
 
-    return text;
+    return atext;
 }
 
-string cuthtml(const string &html, bool cutbrs) {
-    string r, tag;
+string cuthtml(const string &html, int flags) {
+    string r, tag, buf, token;
     int npos, pos, tpos;
 
     for(pos = 0; (npos = html.find("<", pos)) != -1; pos = npos) {
@@ -930,10 +937,34 @@ string cuthtml(const string &html, bool cutbrs) {
 
 	    tag = html.substr(tpos+1, npos-tpos-2);
 	    if(tag.substr(0, 1) == "/") tag.erase(0, 1);
+	    tag = leadcut(trailcut(tag, "/ \n\r"), "/ \n\r");
 
-	    if((tag == "br") || (tag == "BR")) {
-		r += cutbrs ? "\n" : "<br>";
+	    buf = ruscase(tag, "toupper");
+	    token = getword(buf);
+
+	    if(token == "BR") r += (flags & chCutBR) ? "\n" : "<br>";
+		else if((flags & chCutBR) && token == "P") r += "\n\n";
+
+	    if(flags & chLeaveLinks) {
+		getword(tag);
+
+		if(token == "A") {
+		    if((tpos = buf.find("HREF")) != -1)
+		    if((tpos = buf.substr(tpos).find("\"")) != -1) {
+			tag.erase(0, tpos+1);
+			r += "[ href: " + getword(tag, "\"") + " ] ";
+		    }
+
+		} else if(token == "IMG") {
+		    if((tpos = buf.find("SRC")) != -1)
+		    if((tpos = buf.substr(tpos).find("\"")) != -1) {
+			tag.erase(0, tpos+1);
+			r += " [ img: " + getword(tag, "\"") + " ]";
+		    }
+
+		}
 	    }
+
 	} else {
 	    r += html.substr(tpos);
 	    npos = html.size();
@@ -1015,3 +1046,74 @@ char *str_to_utf8(const char *pin) {
     result[i] = '\0';
     return result;
 }
+
+string striprtf(const string &s) {
+    string r, spec;
+    char pre = 0;
+    bool bprint, bspec;
+    int bparen = -1;
+
+    bprint = true;
+    bspec = false;
+
+    for(string::const_iterator i = s.begin(); i != s.end(); ++i) {
+	if(!isalpha(*i) && !isdigit(*i)) bprint = true;
+
+	if(bspec) {
+	    spec += *i;
+
+	    if(spec.size() == 2) {
+		r += (char) hex2int(spec);
+		bspec = false;
+		bprint = true;
+	    }
+
+	} else switch(*i) {
+	    case '{':
+		if(pre != '\\') {
+		    bparen++;
+		    bprint = false;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+		break;
+
+	    case '}':
+		if(pre != '\\') {
+		    bprint = false;
+		    bparen--;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+
+		break;
+
+	    case '\\':
+		if(pre != '\\') {
+		    bprint = false;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+		break;
+
+	    case '\'':
+		if(pre == '\\') {
+		    spec = "";
+		    bspec = true;
+		}
+		break;
+
+	    default:
+		if(bprint && !bparen) {
+		    r += *i;
+		}
+	}
+
+	pre = *i;
+    }
+
+    return leadcut(trailcut(r));
+}
