@@ -15,10 +15,14 @@
 
 #include "conf.h"
 
-#ifdef HAVE_ICONV_H
+#ifdef HAVE_ICONV
 #include <iconv.h>
 #endif
 
+//
+// Gives out a number with the both limits _included_
+//
+
 #define randlimit(l, h) ((int)((float)rand()/(float)RAND_MAX*(float)(h+1-l)+(float)l))
 
 #define SWAPVAL(v1, v2) { \
@@ -104,7 +108,12 @@ string fromutf8(const string &text);
 
 string ruscase(const string &s, const string &mode);
 string siconv(const string &text, const string &fromcs, const string &tocs);
-string cuthtml(const string &html, bool cutbrs = false);
+
+const int chCutBR = 1;
+const int chLeaveLinks = 2;
+
+string cuthtml(const string &html, int flags = 0);
+string striprtf(const string &s);
 
 __KTOOL_BEGIN_C
 
