@@ -1,6 +1,7 @@
 #include "projectname.h"
 #include "motormisc.h"
 #include "motorconfiguration.h"
+#include "kkfsys.h"
 
 #include <sys/stat.h>
 
@@ -16,7 +17,7 @@ projectname::projectname(const string &aname/*, const string arevision*/) {
 projectname::~projectname() {
 }
 
-string projectname::getfname() const {
+string projectname::getregfname() const {
     string r = conf.getprojectdir() + name;
 //    if(!revision.empty()) r += (string) "-" + revision;
     return r;
@@ -55,6 +56,29 @@ void projectname::setrevision(const string arevision) {
     revision = arevision;
 }
 */
+
+bool projectname::convert4() {
+    bool r = false;
+    ifstream f(getregfname().c_str());
+    string rootdir, sect, buf;
+    struct stat st;
+
+    if(r = f.is_open()) {
+	while(getconf(sect, buf, f) && rootdir.empty())
+    	    if(sect == "rootdir") rootdir = buf;
+
+	f.close();
+
+	if(r = (!rootdir.empty() && !stat(rootdir.c_str(), &st))) {
+	    string projfname = rootdir + "/" + name + ".motor";
+	    fmove(getregfname().c_str(), projfname.c_str());
+	    symlink(projfname.c_str(), getregfname().c_str());
+	}
+    }
+
+    return r;
+}
+
 bool projectname::load(const string &aname/*, const string arevision*/) {
     ifstream f;
     string sect, buf;
@@ -64,17 +88,16 @@ bool projectname::load(const string &aname/*, const string arevision*/) {
     name = aname;
 //    revision = arevision;
 
-    if(!stat(getfname().c_str(), &st)) {
-	if(S_ISLNK(st.st_mode)) {
-	    lstat(getfname().c_str(), &st);
-	}
-        if(!S_ISREG(st.st_mode)) {
-	    return false;
-	}
+    if(!lstat(getregfname().c_str(), &st)) {
+	if(!S_ISLNK(st.st_mode))
+	    if(!convert4())
+		return false;
+    } else {
+	return false;
     }
 
     if(!loaded || (aname != name)) {
-	f.open(getfname().c_str());
+	f.open(getregfname().c_str());
 
 	if(loaded = ret = f.is_open()) {
 	    fmodified = false;
@@ -86,6 +109,7 @@ bool projectname::load(const string &aname/*, const string arevision*/) {
 	    }
 
 	    f.close();
+	    loaded = !version.empty() && !templname.empty();
 	}
     }
 
