@@ -28,6 +28,7 @@ int uitagbrowser::mkeys(verticalmenu &m, int k) {
     switch(k) {
         case KEY_BACKSPACE:
 	    if(!uitb.search.empty()) {
+                uitb.modified = true;
 		uitb.search = oldsearch.substr(0, oldsearch.size()-1);
 	    }
             break;
@@ -40,6 +41,10 @@ int uitagbrowser::mkeys(verticalmenu &m, int k) {
 	    uitb.changemode();
 	    break;
 
+	case KEY_F(3):
+	    uitb.changefilter();
+	    break;
+
 	case CTRL('r'):
 	    uitb.reload();
 	    break;
@@ -99,6 +104,70 @@ void uitagbrowser::changemode() {
     }
 }
 
+int uitagbrowser::multiplekeys(verticalmenu &m, int k) {
+    switch(k) {
+	case ' ':
+	case 'x':
+	case 'X':
+	    return -2;
+    }
+    return -1;
+}
+
+void uitagbrowser::changefilter() {
+    bool finished = false;
+    int saveelem, pos;
+    char c;
+    bool iter;
+    string nfilter = filter;
+    vector<pair<char, string > >::const_iterator ik;
+
+    verticalmenu cm(uiconf.getcolor(cp_menufr), uiconf.getcolor(cp_menusel));
+    cm.setwindow(textwindow(w.x1, w.y1, w.x1+20, 0, uiconf.getcolor(cp_menufr)));
+
+    cm.otherkeys = &multiplekeys;
+
+    while(!finished) {
+	saveelem = cm.getpos();
+	cm.clear();
+
+	for(ik = project.tagnamebegin(); ik != project.tagnameend(); ++ik) {
+	    cm.additemf(0, ik->first, "[%c] %s", (nfilter.find(ik->first) != -1) ? 'x' : ' ',
+		ik->second.c_str());
+	}
+
+	cm.scale();
+	cm.setpos(saveelem);
+
+	switch(cm.open()) {
+	    case -2:
+		iter = false;
+		c = (char) (int) cm.getref(cm.getpos());
+
+		while((pos = nfilter.find(c)) != -1) {
+		    nfilter.erase(pos, 1);
+		    iter = true;
+		}
+
+		if(!iter)
+		    nfilter += c;
+		break;
+
+	    default:
+		if(!nfilter.empty())
+		    finished = true;
+		break;
+	}
+    }
+
+    cm.close();
+
+    if((cm.getlastkey() != KEY_ESC) && (filter != nfilter)) {
+	filter = nfilter;
+	makemenu(m);
+    }
+}
+
 void uitagbrowser::reload() {
     ui.autosave();
     tagbrowser.clear();
@@ -111,13 +180,13 @@ void uitagbrowser::makemenu(verticalmenu &m) {
     string buf, desc, skind;
     vector<motortagbrowser::record>::iterator it;
 
-    tags = tagbrowser.fetch(search);
+    tags = tagbrowser.fetch(search, filter);
     m.clear();
 
     for(it = tags.begin(); it != tags.end(); it++) {
         buf = " " + it->getname();
 	desc = it->getbrowsertagname();
-	
+
         if((skind = it->getinfo()).size() && (skind[skind.size()-1] != ':')) {
             if(desc.size()) desc += ", ";
             desc += skind;
@@ -132,7 +201,7 @@ void uitagbrowser::makemenu(verticalmenu &m) {
         m.additem(0, it-tags.begin(), buf);
     }
 
-    ui.log(_("Symbols: type to find, ~Del~ clear, ~F2~ set scope, ~^R~ reload, ~Enter~ go to source"));
+    ui.log(_("Symbols: type to find, ~Del~ clear, ~F2~ scope, ~F3~ filter, ~^R~ reload, ~Enter~ source"));
     m.redraw();
     input();
 }
